#version 330 core

layout (triangles) in;
layout (triangle_strip) out;
layout (max_vertices = 4) out;

out vec3 f_position;
out vec2 f_texCoord;
out vec3 f_normal;

uniform mat4 view;
uniform mat4 viewProj;
uniform vec3 cameraPos;

uniform vec3 bilbDims;

void main()
{
	float width = bilbDims.x;
	float height = bilbDims.y;

	vec3 pos = gl_in[0].gl_Position.xyz;
	vec3 toCamera = normalize(cameraPos - pos);
    vec3 up = vec3(0.0, 1.0, 0.0);
    vec3 right = cross(toCamera, up) * width;
	
	f_normal = vec3(0.0f, 0.0f, 1.0f);
	
	pos -= (right * 0.5);
    gl_Position = viewProj * vec4(pos, 1.0);
    f_texCoord = vec2(0.0, 1.0);
	f_position = vec3(view * vec4(pos, 1.0f));
    EmitVertex();

    pos.y += height;
    gl_Position = viewProj * vec4(pos, 1.0);
    f_texCoord = vec2(0.0, 0.0);
	f_position = vec3(view * vec4(pos, 1.0f));
    EmitVertex();

    pos.y -= height;
    pos += right;
    gl_Position = viewProj * vec4(pos, 1.0);
    f_texCoord = vec2(1.0, 1.0);
	f_position = vec3(view * vec4(pos, 1.0f));
    EmitVertex();

    pos.y += height;
    gl_Position = viewProj * vec4(pos, 1.0);
    f_texCoord = vec2(1.0, 0.0);
	f_position = vec3(view * vec4(pos, 1.0f));
    EmitVertex();

    EndPrimitive();
}