#version 330 core

out vec4 outColor;

struct PointLight
{
	vec3 color;
	float ambientIntensity;
	float diffuseIntensity;
	float constant;
	float linear;
	float quadratic;
	float radius;
};

uniform vec2 gScreenSize;

uniform sampler2D gAlbedo;
uniform sampler2D gPosition;
uniform sampler2D gNormals;

uniform mat4 view;
uniform vec3 worldPosition;
uniform PointLight pointLight;


vec2 calcTexCoords();
vec4 calcPointLight(vec3 viewPosSample, vec3 normalSample, float shininessSample);
vec4 calcLightContribution(vec3 lightDir, vec3 normal, vec3 viewPos, float shininess);

void main()
{
	vec2 texCoords = calcTexCoords();
	
	vec3 diffSample = texture(gAlbedo, texCoords).xyz;
	float specSample = texture(gAlbedo, texCoords).w;
	vec3 normalSample = normalize(texture(gNormals, texCoords).xyz);
	vec3 positionSample = texture(gPosition, texCoords).xyz;
	
	//vec3 showbuffer = abs(vec3(positionSample.z, positionSample.z, positionSample.z));
    outColor = vec4(diffSample, 1.0f) * calcPointLight(positionSample, normalSample, specSample); //diffSample * max(0.0, dot(normalSample, normalize(vec3(1.0, 1.0, 0.0)))) + diffSample * 0.1;
}

vec4 calcPointLight(vec3 viewPosSample, vec3 normalSample, float shininessSample)
{
	vec3 lightPos = (view * vec4(worldPosition, 1.0f)).xyz;
	vec3 lightDir = viewPosSample.xyz - lightPos;
	float dist = length(lightDir);
	lightDir = normalize(lightDir);
	vec4 lightContrib = calcLightContribution(lightDir, normalSample, viewPosSample, shininessSample);
	float attenuation = pointLight.constant +
						pointLight.linear * dist + 
						pointLight.quadratic * dist * dist;
	attenuation = max(1.0f, attenuation);
	return lightContrib / attenuation;
}

vec4 calcLightContribution(vec3 lightDir, vec3 normal, vec3 viewPos, float shininess)
{
	vec4 ambientColor = vec4(pointLight.color * pointLight.ambientIntensity, 1.0f);
	
	float diffuseFactor = max(0.0f, dot(normal, -lightDir));
	vec4 diffuseColor = vec4(diffuseFactor * pointLight.color * pointLight.diffuseIntensity, 1.0f);
	
	vec3 vertToEye = normalize(-viewPos);
	vec3 lightReflect = normalize(reflect(lightDir, normal));
	float specFactor = max(0.0f, dot(vertToEye, lightReflect));
	specFactor = pow(specFactor, 32);
	vec4 specularColor = vec4(specFactor * pointLight.color * pointLight.diffuseIntensity, 1.0f);
	
	return ambientColor + diffuseColor + specularColor;
}


vec2 calcTexCoords()
{
   return gl_FragCoord.xy / gScreenSize.xy;
}